Encoding: UTF-8
Package: piecepackr
Type: Package
Title: Board Game Graphics
Version: 1.16.1
Description: Functions to make board game graphics with the 'ggplot2', 'grid', 'rayrender', 'rayvertex', and 'rgl' packages.  Specializes in game diagrams, animations, and "Print & Play" layouts for the 'piecepack' <https://www.ludism.org/ppwiki> but can make graphics for other board game systems.  Includes configurations for several public domain game systems such as checkers, (double-18) dominoes, go, 'piecepack', playing cards, etc.
Authors@R: c(person("Trevor L.", "Davis", role=c("aut", "cre"),
             email="trevor.l.davis@gmail.com",
             comment = c(ORCID = "0000-0001-6341-4639")),
             person("Linux Foundation", role="dtc",
                    comment = 'Uses some data from the "SPDX License List" <https://github.com/spdx/license-list-XML>'),
             person("Delapouite <https://delapouite.com/>", role="ill",
                     comment = 'Meeple shape extracted from "Meeple icon" <https://game-icons.net/1x1/delapouite/meeple.html> / "CC BY 3.0" <https://creativecommons.org/licenses/by/3.0/>'),
             person("Creative Commons", role="ill",
                    comment = '`save_print_and_play()` uses "license badges" from Creative Commons to describe the generated print-and-play file\'s license'))
License: MIT + file LICENSE
URL: https://trevorldavis.com/piecepackr/ (blog),
        https://trevorldavis.com/R/piecepackr/ (pkgdown),
        https://groups.google.com/forum/#!forum/piecepackr (forum)
BugReports: https://github.com/piecepackr/piecepackr/issues
LazyData: true
LazyLoad: yes
Imports: affiner (>= 0.3.1), grid, gridGeometry, grImport2, grDevices,
        purrr, jpeg, png, R6, rlang, stringr, tibble, tools, utils
Suggests: animation (>= 2.7), ggplot2, gifski, gridpattern, magick,
        pdftools, rayrender (>= 0.34.3), rayvertex (>= 0.10.4), readobj
        (>= 0.4.0), rgl (>= 0.106.8), scales (>= 0.5.0), svglite (>=
        2.2.2), systemfonts, testthat, tweenr, vdiffr, xmpdf (>=
        0.1.1), XML (>= 3.99-0.9)
Config/testthat/edition: 3
Config/roxygen2/version: 8.0.0
NeedsCompilation: no
Packaged: 2026-05-12 04:08:55 UTC; trevorld
Author: Trevor L. Davis [aut, cre] (ORCID:
    <https://orcid.org/0000-0001-6341-4639>),
  Linux Foundation [dtc] (Uses some data from the "SPDX License List"
    <https://github.com/spdx/license-list-XML>),
  Delapouite <https://delapouite.com/> [ill] (Meeple shape extracted from
    "Meeple icon" <https://game-icons.net/1x1/delapouite/meeple.html> /
    "CC BY 3.0" <https://creativecommons.org/licenses/by/3.0/>),
  Creative Commons [ill] (`save_print_and_play()` uses "license badges"
    from Creative Commons to describe the generated print-and-play
    file's license)
Maintainer: Trevor L. Davis <trevor.l.davis@gmail.com>
Depends: R (>= 2.10)
Repository: CRAN
Date/Publication: 2026-05-12 05:40:02 UTC
Built: R 4.6.0; ; 2026-05-12 07:08:59 UTC; unix
