# Generated by roxygen2: do not edit by hand

S3method(as.list,pp_cfg)
S3method(grobCoords,basic_piece_side)
S3method(grobCoords,basic_projected_token)
S3method(grobCoords,coords_xyl)
S3method(grobCoords,general_projected_token)
S3method(grobCoords,holed_board)
S3method(grobCoords,pmap_piece)
S3method(grobCoords,pp_grobCoords)
S3method(grobCoords,pp_ps_transformation)
S3method(grobCoords,pyramid_top)
S3method(grobPoints,pmap_piece)
S3method(grobPoints,pp_grobCoords)
S3method(makeContent,alquerque_board)
S3method(makeContent,basic_ellipsoid)
S3method(makeContent,basic_grob_with_bleed)
S3method(makeContent,basic_piece_side)
S3method(makeContent,basic_projected_die)
S3method(makeContent,basic_projected_token)
S3method(makeContent,card_side)
S3method(makeContent,checkered_board)
S3method(makeContent,checkers)
S3method(makeContent,d4_die)
S3method(makeContent,domino_side)
S3method(makeContent,face_card_side)
S3method(makeContent,fudge_die)
S3method(makeContent,general_projected_token)
S3method(makeContent,geom_piece_grob)
S3method(makeContent,gridlines)
S3method(makeContent,hexlines)
S3method(makeContent,holed_board)
S3method(makeContent,joker_card_side)
S3method(makeContent,lined_board)
S3method(makeContent,mat)
S3method(makeContent,morris_board)
S3method(makeContent,piece)
S3method(makeContent,pipped)
S3method(makeContent,pp_cropmark)
S3method(makeContent,pp_crosshair)
S3method(makeContent,pp_picture)
S3method(makeContent,pp_polyclip)
S3method(makeContent,pp_ps_transformation)
S3method(makeContent,projected_ellipsoid)
S3method(makeContent,projected_holed_board)
S3method(makeContent,projected_pyramid_side)
S3method(makeContent,projected_pyramid_top)
S3method(makeContent,projected_rpg_die)
S3method(makeContent,pyramid_top)
S3method(makeContext,basic_piece_side)
S3method(makeContext,checkers)
S3method(makeContext,geom_piece_grob)
S3method(makeContext,gridlines)
S3method(makeContext,hexlines)
S3method(makeContext,mat)
S3method(makeContext,pp_cropmark)
export(AA_to_R)
export(R_to_AA)
export(R_x)
export(R_y)
export(R_z)
export(aabb_piece)
export(aes_piece)
export(animate_piece)
export(as_pp_cfg)
export(basicPieceGrob)
export(breaks_counting)
export(circledSegmentsCrosshairGrob)
export(cleave)
export(cropmarkGrob)
export(crosshairGrob)
export(file2grob)
export(game_systems)
export(geom_piece)
export(get_embedded_font)
export(grid.cropmark)
export(grid.crosshair)
export(grid.piece)
export(has_font)
export(inch)
export(install_ppverse)
export(is_color_invisible)
export(is_pp_cfg)
export(label_counting)
export(label_letter)
export(marbles_transform)
export(op_transform)
export(picturePieceGrobFn)
export(piece)
export(piece3d)
export(pieceGrob)
export(piece_mesh)
export(pkgs_ppverse)
export(pmap_piece)
export(pp_cfg)
export(pp_shape)
export(previewLayoutGrob)
export(pyramidTopGrob)
export(render_piece)
export(save_ellipsoid_obj)
export(save_peg_doll_obj)
export(save_piece_images)
export(save_piece_obj)
export(save_print_and_play)
export(scale_x_piece)
export(scale_y_piece)
export(segmentsCrosshairGrob)
export(squaredSegmentsCrosshairGrob)
export(squaresCrosshairGrob)
export(to_degrees)
export(to_hexpack)
export(to_r)
export(to_radians)
export(to_subpack)
export(to_t)
export(to_x)
export(to_y)
import(grid)
importFrom(R6,R6Class)
importFrom(affiner,as_coord2d)
importFrom(affiner,as_coord3d)
importFrom(affiner,as_ellipse2d)
importFrom(affiner,as_plane3d)
importFrom(affiner,as_polygon2d)
importFrom(affiner,as_segment2d)
importFrom(affiner,convex_hull2d)
importFrom(affiner,degrees)
importFrom(affiner,distance2d)
importFrom(affiner,distance3d)
importFrom(affiner,has_overlap2d)
importFrom(affiner,is_angle)
importFrom(affiner,painter_depth)
importFrom(affiner,painter_order)
importFrom(affiner,regular_ngon_polygon2d)
importFrom(grDevices,col2rgb)
importFrom(grDevices,dev.capabilities)
importFrom(grDevices,dev.list)
importFrom(grDevices,dev.new)
importFrom(grDevices,devAskNewPage)
importFrom(grDevices,rgb)
importFrom(rlang,"%||%")
importFrom(rlang,.data)
importFrom(rlang,abort)
importFrom(rlang,check_dots_empty)
importFrom(rlang,inform)
importFrom(rlang,warn)
importFrom(stringr,str_count)
importFrom(stringr,str_glue)
importFrom(stringr,str_pad)
importFrom(stringr,str_split)
importFrom(tibble,tibble)
importFrom(utils,hasName)
importFrom(utils,head)
importFrom(utils,packageDescription)
importFrom(utils,packageVersion)
importFrom(utils,tail)
