The "svgz" files in `extdata/badges` are Creative Commons License "button marks" from https://github.com/creativecommons/cc-assets made available under the following license (https://creativecommons.org/policies):
  
  ```
  Creative Commons License Buttons and Icons: Creative Commons licenses the use of its button marks that describe a particular legal tool and its icon marks that describe a key license element, such as BY, NC, ND, and SA, on the conditions that you use the mark solely to describe the Creative Commons legal tool that applies to a particular work and, in a manner reasonable to the medium and context, include the URI or hyperlink to the relevant Commons deed on the Creative Commons server.
  ```

  They are solely used by `save_print_and_play()` to describe
  the Creative Commons legal tool that applies to the generated print-and-play file.
  Additionally `save_print_and_play()` also includes in the generated print-and-play file
  a URI to the relevant Commons deed on the Creative Commons server.
