#' @section Package options:
#' The following piecepackr function arguments may be set globally via [base::options()]:
#'  \describe{
#'    \item{piecepackr.at.inform}{If `FALSE` turns off messages when affine transformation support not detected in active graphics device.}
#'    \item{piecepackr.cfg}{Sets a new default for the `cfg` argument}
#'    \item{piecepackr.check.cairo}{If `FALSE` don't check the version of cairo}
#'    \item{piecepackr.default.units}{Sets a new default for the `default.units` argument}
#'    \item{piecepackr.envir}{Sets a new default for the `envir` argument}
#'    \item{piecepackr.fs.inform}{If `FALSE` turns off messages when stroking and filling path support not detected in active graphics device.}
#'    \item{piecepackr.metadata.inform}{If `FALSE` turns off messages when support for embedding metadata not detected.}
#'    \item{piecepackr.op_angle}{Sets a new default for the `op_angle` argument}
#'    \item{piecepackr.op_scale}{Sets a new default for the `op_scale` argument}
#'    \item{piecepackr.rgr.inform}{If `FALSE` turns off messages when radial gradient support not detected in active graphics device.}
#'    \item{piecepackr.trans}{Sets a new default for the `trans` argument}
#'  }
#' @seealso Useful links:
#'
#'   * blog: \url{https://trevorldavis.com/piecepackr/}
#'   * pkgdown: \url{https://trevorldavis.com/R/piecepackr/}
#'   * forum: \url{https://groups.google.com/forum/#!forum/piecepackr}
#'   * Report bugs: \url{https://github.com/piecepackr/piecepackr/issues}
#'
"_PACKAGE"
