# #' Deprecated functions
# #'
# #' These functions are Deprecated in this release of piecepackr,
# #' they will be marked as Defunct and removed in a future version.
# #'
# #' @name piecepackr-deprecated
# NULL
