# removed `style` argument errors

    Code
      game_systems(style = "dejavu3d")
    Condition
      Error in `game_systems()`:
      ! `...` must be empty.
      x Problematic argument:
      * style = "dejavu3d"

